<?php

namespace Drupal\social_feeds_block\Plugin\Block;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'Facebook feeds' block.
 *
 * @Block(
 *   id = "fb_feeds_block",
 *   admin_label = @Translation("Facebook social feeds")
 * )
 */
class FacebookFeedsBlock extends BlockBase implements ContainerFactoryPluginInterface {


  /**
  * The Facebook graph domain.
  */
  const GRAPH_DOMAIN = 'graph.facebook.com';

  /**
 * The Graph API version used.
 */
  const GRAPH_API_VERSION = 'v15.0';


  /**
   * Contains the configuration object factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configfactory;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory, ClientInterface $http_client, LoggerChannelFactoryInterface $logger_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configfactory = $config_factory;
    $this->httpClient = $http_client;
    $this->logger = $logger_factory->get('social_feeds_block');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration, $plugin_id, $plugin_definition, $container->get('config.factory'), $container->get('http_client'), $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->configuration;

    $form['social_feeds_block_fb'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Facebook credential'),
      '#weight' => 50,
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    ];

    $form['social_feeds_block_fb']['fb_page_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Facebook page name'),
      '#default_value' => $config['fb_page_name'],
      '#description' => $this->t('e.g. If your Facebook page URL is @facebook.', ['@facebook' => 'https://www.facebook.com/YOUR_PAGE_NAME']),
      '#maxlength' => 255,
      '#required' => TRUE,
    ];

    $form['social_feeds_block_fb']['fb_app_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Facebook app id'),
      '#default_value' => $config['fb_app_id'],
      '#description' => $this->t('App Id and Secret Key is available at @facebook', ['@facebook' => 'https://developers.facebook.com/apps/<YOUR-APP-ID>/settings/basic/']),
      '#maxlength' => 255,
      '#required' => TRUE,
    ];

    $form['social_feeds_block_fb']['fb_secret_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Facebook secret key'),
      '#default_value' => $config['fb_secret_id'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];
    $form['social_feeds_block_fb']['fb_user_token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Facebook access token'),
      '#default_value' => $config['fb_user_token'],
      '#description' => $this->t('You can generate this at <a href=":facebook" target="_blank">facebook</a>
        <ul>
          <li>Step1 - Select the appropriate App under <em>Facebook App</em></li>
          <li>Step2 - Then from <em>User or Page</em> select your listed page from <em>Page Access Tokens</em></li>
        </ul>
        ', [':facebook' => 'https://developers.facebook.com/tools/explorer/']),
      '#size' => 60,
      '#maxlength' => 255,
      '#required' => TRUE,
    ];
    $form['social_feeds_block_fb']['fb_no_feeds'] = [
      '#type' => 'number',
      '#title' => $this->t('Number of feeds'),
      '#default_value' => $config['fb_no_feeds'],
      '#maxlength' => 2,
      '#required' => TRUE,
    ];

    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $fb_config = $form_state->getValue('social_feeds_block_fb');
    $this->configuration['fb_page_name'] = $fb_config['fb_page_name'];
    $this->configuration['fb_app_id'] = $fb_config['fb_app_id'];
    $this->configuration['fb_secret_id'] = $fb_config['fb_secret_id'];
    $this->configuration['fb_user_token'] = $fb_config['fb_user_token'];
    $this->configuration['fb_no_feeds'] = $fb_config['fb_no_feeds'];

  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $fb_feeds = '';
    $error_message = '';

    $creds = $this->configuration;
    $fb_page_name = $creds['fb_page_name'];
    $fb_app_id = $creds['fb_app_id'];
    $fb_secret_id = $creds['fb_secret_id'];
    $fb_no_feeds = $creds['fb_no_feeds'];

    $fb_feed_url = $this->getFacebookFeedUrl($fb_no_feeds);

    if (isset($fb_page_name) && isset($fb_feed_url) && isset($fb_app_id) && isset($fb_secret_id)) {
      try {
        $response = $this->httpClient->get($fb_feed_url, ['headers' => ['Accept' => 'text/plain']]);
        $data = $response->getBody();
        $obj = json_decode($data, TRUE);
        $fb_feeds = $obj['data'];
      }
      catch (\Exception $e) {
        watchdog_exception('social_feeds_block', $e);
      }
    }
    else {
      $error_message = $this->t('API Credentials are Missing.');
    }

    return [
      '#theme' => 'fb_social_feeds_block',
      '#data' => $fb_feeds,
      '#error_message' => $error_message,
    ];
  }

  /**
   * Creates the Facebook feed URL.
   *
   * @param int $num_posts
   *   The number of posts to return.
   *
   * @return string
   *   The feed URL with the appended fields to retrieve.
   */
  protected function getFacebookFeedUrl($num_posts): string {

    $fields = [
      'picture',
      'permalink_url',
      'status_type',
      'message',
    ];
    // To generate Long-Lived access token is generated .
    $this->defaultAccessToken();
    $url = Url::fromUri('https://' . self::GRAPH_DOMAIN)->toString();
    $config = $this->configfactory->get('config.facebook_social_feeds_block');
    $return = $url . '/' . $config->get('page_id') . '/feed?access_token=' . $config->get('page_permanent_token') . '&fields=' . implode(',', $fields) . '&limit=' . $num_posts;

    return $return;
  }

  /**
   * Generates the Facebook access token.
   *
   * @return string
   *   The access token.
   */
  protected function defaultAccessToken(): string {
    $config = $this->configfactory->getEditable('config.facebook_social_feeds_block');
    $permanent_token = $config->get('page_permanent_token');
    if (empty($permanent_token)) {
      $args = [
        'user_token' => $config->get('fb_user_token'),
        'app_id' => $config->get('fb_app_id'),
        'app_secret' => $config->get('fb_secret_id'),
        'page_name' => $config->get('fb_page_name'),
      ];

      $url = Url::fromUri('https://' . self::GRAPH_DOMAIN . '/' . self::GRAPH_API_VERSION)->toString();
      $client = $this->httpClient;
      // Token.
      try {
        $request = $client->request('GET', $url . "/oauth/access_token?grant_type=fb_exchange_token&client_id={$args['app_id']}&client_secret={$args['app_secret']}&fb_exchange_token={$args['user_token']}");
        $request = json_decode($request->getBody()->getContents());
        $long_token = $request->access_token;
      }
      catch (\Exception $e) {
        watchdog_exception('social_feeds_block', $e);
      }
      // User ID.
      try {
        $request = $client->request('GET', $url . "/me?access_token=$long_token");
        $request = json_decode($request->getBody()->getContents());
        $account_id = $request->id;
      }
      catch (\Exception $e) {
        watchdog_exception('social_feeds_block', $e);
      }
      // Page ID.
      try {
        $request = $client->request('GET', $url . "/{$args['page_name']}?fields=id&access_token=$long_token");
        $request = json_decode($request->getBody()->getContents());
        $page_id = $request->id;
        $config->set('page_id', $page_id)->save();
      }
      catch (\Exception $e) {
        watchdog_exception('social_feeds_block', $e);
      }
      // Genearating Permanent Token.
      try {
        $request = $client->request('GET', $url . "/$account_id/accounts?access_token=$long_token");
        $request = json_decode($request->getBody()->getContents());
        foreach ($request->data as $response_data) {
          if ($response_data->id == $page_id) {
            $config->set('page_permanent_token', $response_data->access_token)->save();
            return $response_data->access_token;
          }
        }
      }
      catch (\Exception $e) {
        watchdog_exception('social_feeds_block', $e);
      }
    }
    return $permanent_token;
  }

}
