<?php

namespace Drupal\social_feeds_block\Plugin\Block;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use EspressoDev\InstagramBasicDisplay\InstagramBasicDisplay;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Component\Render\FormattableMarkup;

/**
 * Provides a 'Instagram feeds' block.
 *
 * @Block(
 *   id = "instagram_feeds_block",
 *   admin_label = @Translation("Instagram social feeds")
 * )
 */
class InstagramFeedsBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Contains the configuration object factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configfactory;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configfactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration, $plugin_id, $plugin_definition, $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->configuration;
    $redirect_uri = Url::fromRoute('social_feeds_block.instagram_auth', [], ['absolute' => TRUE])->toString();

    $form['header']['#markup'] = new FormattableMarkup('To get the App ID and App Secret follow the  steps 1-3 <a href=":site" target=":blank">here</a>. Then add them here and save the page. Once the credentials are saved, a link to generate the Access Token will appear.', [
      ':site' => "https://developers.facebook.com/docs/instagram-basic-display-api/getting-started",
      ':blank' => "_blank",
    ]);
    $form['instagram_social_feeds'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Instagram credential'),
      '#weight' => 50,
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    ];

    $form['instagram_social_feeds']['insta_app_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('App id'),
      '#default_value' => $config['insta_app_id'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];

    $form['instagram_social_feeds']['insta_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('App secret'),
      '#default_value' => $config['insta_secret'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];
    $form['instagram_social_feeds']['redirect_uri'] = [
      '#type' => 'item',
      '#title' => $this->t('Redirect uri'),
      '#markup' => $redirect_uri,
      '#default_value' => $redirect_uri,
    ];

    $token_message = $this->t('Once the app ID and secret Key have been saved, a link to generate the access Key will appear.');
    if ($config['insta_app_id']) {
      $instagram = new InstagramBasicDisplay([
        'appId' => $config['insta_app_id'],
        'appSecret' => $config['insta_secret'],
        'redirectUri' => $redirect_uri,
      ]);

      $token_message = $this->t('<a href=":this" target="_blank">Login with Instagram to generate the access token</a>', [
        ':this' => Url::fromUri($instagram->getLoginUrl())->toString(),
      ]);
    }

    $form['instagram_social_feeds']['insta_access_token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Access token'),
      '#field_prefix' => '<div>' . $token_message . '</div>',
      '#description' => $this->t('Token will automatically be renewed before the current one expires in 60 days.'),
      '#default_value' => $config['insta_access_token'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];

    $form['instagram_social_feeds']['insta_pic_counts'] = [
      '#type' => 'number',
      '#title' => $this->t('Post count'),
      '#default_value' => $config['insta_pic_counts'],
      '#maxlength' => 2,
      '#required' => TRUE,
    ];

    $form['instagram_social_feeds']['insta_post_link'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show post url'),
      '#default_value' => $config['insta_post_link'],
    ];
    $form['instagram_social_feeds']['insta_video_thumbnail'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show video thumbnails instead of videos'),
      '#default_value' => $config['insta_video_thumbnail'],
    ];

    $form['instagram_social_feeds']['insta_child_show'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show all image/media (If a post have multi files)'),
      '#default_value' => $config['insta_child_show'],
    ];

    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {

    $instagram_config = $form_state->getValue('instagram_social_feeds');
    $this->configuration['insta_app_id'] = $instagram_config['insta_app_id'];
    $this->configuration['insta_secret'] = $instagram_config['insta_secret'];
    $this->configuration['insta_access_token'] = $instagram_config['insta_access_token'];
    $this->configuration['insta_pic_counts'] = $instagram_config['insta_pic_counts'];
    $this->configuration['insta_post_link'] = $instagram_config['insta_post_link'];
    $this->configuration['insta_video_thumbnail'] = $instagram_config['insta_video_thumbnail'];
    $this->configuration['insta_child_show'] = $instagram_config['insta_child_show'];

  }

  /**
   * {@inheritdoc}
   */
  public function build() {

    $creds = $this->configuration;
    // Refresh the long-lived Access Token.
    $this->regenerateAccessToken($creds);
    $items = $this->getPost($creds);

    return [
      '#theme' => 'item_list',
      '#items' => $items,
    ];
  }

  /**
   * Getting posts.
   */
  public function getPost($creds) {

    $insta_access_token = $creds['insta_access_token'];
    $insta_pic_counts = $creds['insta_pic_counts'];
    // Settings.
    $settigs['insta_child_show'] = $creds['insta_child_show'];
    $settigs['insta_post_link'] = $creds['insta_post_link'];
    $settigs['insta_video_thumbnail'] = $creds['insta_video_thumbnail'];
    // Insta.
    $instagram = new InstagramBasicDisplay($insta_access_token);
    $posts = $instagram->getUserMedia('me', $insta_pic_counts);
    foreach ($posts->data as $post) {

      $items[] = [
        '#theme' => 'instagram_social_feeds_block_' . strtolower($post->media_type),
        '#post' => $post,
        '#config' => $settigs,
      ];
    }
    return $items;

  }

  /**
   * Update the access token with a "long-lived" one.
   *
   * @throws \EspressoDev\InstagramBasicDisplay\InstagramBasicDisplayException
   */
  protected function regenerateAccessToken($creds) {

    // 50 Days.
    $days_later = 50 * 24 * 60 * 60;

    // Exit if the token doesn't need updating.
    if (empty($creds['insta_access_token_date']) || ($creds['insta_access_token_date'] + $days_later) > time()) {
      return;
    }

    // Update the token.
    $instagram = new InstagramBasicDisplay([
      'appId' => $creds['insta_app_id'],
      'appSecret' => $creds['insta_secret'],
      'redirectUri' => $this->currentRequest->getSchemeAndHttpHost() . Url::fromRoute('social_feeds_block.instagram_auth')->toString(),
    ]);

    // Refresh this token.
    $token = $instagram->refreshToken($creds['insta_access_token'], TRUE);

    if ($token) {
      $creds->set('insta_access_token', $token);
      $creds->set('insta_access_token_date', time());
      $creds->save();
    }
  }

}
