<?php

namespace Drupal\social_feeds_block\Plugin\Block;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Url;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'Pinterest feeds' block.
 *
 * @Block(
 *   id = "pinterest_feeds_block",
 *   admin_label = @Translation("Pinterest social feeds")
 * )
 */
class PinterestFeedsBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The Pinterest API domain.
*/
  const PINTEREST_DOMAIN = 'api.pinterest.com';

  /**
   * The Pinterest API version used.
   */
  const PINTEREST_API_VERSION = 'v3';

  /**
   * Contains the configuration object factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configfactory;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory, ClientInterface $http_client, LoggerChannelFactoryInterface $logger_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configfactory = $config_factory;
    $this->httpClient = $http_client;
    $this->logger = $logger_factory->get('social_feeds_block');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('http_client'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {

    $config = $this->configuration;
    $form['pinterest_social_feeds'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Pinterest credential'),
      '#weight' => 50,
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    ];
    $form['pinterest_social_feeds']['pinterest_user_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Pinterest user name'),
      '#default_value' => $config['pinterest_user_name'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];
    $form['pinterest_social_feeds']['pinterest_counts'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Pinterest count'),
      '#default_value' => $config['pinterest_counts'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $pinterest_config = $form_state->getValue('pinterest_social_feeds');
    $this->configuration['pinterest_user_name'] = $pinterest_config['pinterest_user_name'];
    $this->configuration['pinterest_counts'] = $pinterest_config['pinterest_counts'];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $feed_pin_data = '';
    $error_message = '';

    $pinterest_user_name = $this->configuration['pinterest_user_name'];
    $showCount = $this->configuration['pinterest_counts'];
    if (isset($pinterest_user_name)) {
      try {
        $url = Url::fromUri('https://' . self::PINTEREST_DOMAIN . '/' . self::PINTEREST_API_VERSION . '/pidgets/users/' . $pinterest_user_name . '/pins')->toString();

        $response = $this->httpClient->get($url, ['headers' => ['Accept' => 'text/json']]);
        $feeds = $response->getBody()->getContents();

        $feeds = json_decode($feeds, TRUE);
        $feed_pin_data = array_slice($feeds['data']['pins'], 0, $showCount);
      }
      catch (\Exception $e) {
        watchdog_exception('social_feeds_block', $e);
      }
      if ($feeds['status'] != 'success') {
        $error_message = $this->t('Wrong pinterest User Name.');
      }
    }
    else {
      $error_message = $this->t("pinterest User Name Missing.");
    }

    return [
      '#theme' => 'pinterest_social_feeds_block',
      '#data' => $feed_pin_data,
      '#error_message' => $error_message,
    ];
  }

}
