<?php

namespace Drupal\social_feeds_block\Plugin\Block;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Abraham\TwitterOAuth\TwitterOAuth;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'Twitter feeds' block.
 *
 * @Block(
 *   id = "twitter_feeds_block",
 *   admin_label = @Translation("Twitter social feeds")
 * )
 */
class TwitterFeedsBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Contains the configuration object factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configfactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configfactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration, $plugin_id, $plugin_definition, $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->configuration;
    $form['twitter_social_feeds'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Twitter credential'),
      '#weight' => 50,
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    ];

    $form['twitter_social_feeds']['tw_api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Twitter api Key'),
      '#default_value' => $config['tw_api_key'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];

    $form['twitter_social_feeds']['tw_api_secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Twitter api secret key'),
      '#default_value' => $config['tw_api_secret_key'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];

    $form['twitter_social_feeds']['tw_access_token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Twitter access token'),
      '#default_value' => $config['tw_access_token'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];

    $form['twitter_social_feeds']['tw_access_token_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Twitter access token secret'),
      '#default_value' => $config['tw_access_token_secret'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];

    $form['twitter_social_feeds']['tw_counts'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Tweetes count'),
      '#default_value' => $config['tw_counts'],
      '#maxlength' => 255,
      '#required' => TRUE,
    ];

    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {

    $twitter_config = $form_state->getValue('twitter_social_feeds');
    $this->configuration['tw_api_key'] = $twitter_config['tw_api_key'];
    $this->configuration['tw_api_secret_key'] = $twitter_config['tw_api_secret_key'];
    $this->configuration['tw_access_token'] = $twitter_config['tw_access_token'];
    $this->configuration['tw_access_token_secret'] = $twitter_config['tw_access_token_secret'];
    $this->configuration['tw_counts'] = $twitter_config['tw_counts'];

  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $error_message = '';

    $creds = $this->configuration;
    $tw_counts = $creds['tw_counts'];
    $twitterObj = $this->twitterAuth($creds);
    $tweets = $this->twitterGetPosts($twitterObj, $tw_counts);

    if ($tweets->errors) {
      $error_message = $tweets->errors[0]->message;
    }

    return [
      '#theme' => 'twitter_social_feeds_block',
      '#data' => $tweets,
      '#error_message' => $error_message,
    ];
  }

  /**
   * Sets the Twitter Connection.
   */
  public function twitterAuth($creds) {
    $api_key = $creds['tw_api_key'];
    $api_secret_key = $creds['tw_api_secret_key'];
    $access_token = $creds['tw_access_token'];
    $access_token_secret = $creds['tw_access_token_secret'];

    return new TwitterOAuth($api_key, $api_secret_key, $access_token, $access_token_secret);

  }

  /**
   * Retrieves Tweets from the given accounts home page.
   *
   * @param object $twitterObj
   *   An Object of Twitter.
   * @param int $count
   *   The number of posts to return.
   *
   * @return array
   *   An array of posts.
   */
  public function twitterGetPosts(object $twitterObj, $count) {
    $result = $twitterObj->get('statuses/user_timeline', [
      'count' => $count,
      'tweet_mode' => 'extended',
    ]);
    return $result;
  }

}
